/*
 * Component:  common
 *
 * Purpose:
 *	    Header file for image memory manager abstract base class
 *
 * Last Updated     : $Author$
 * Update Date Time : $Date$
 * RCS File Name    : $RCSfile$
 * RCS Revision     : $Revision$
 * Locked by        : $Locker$
 * Symbolic name    : $Name$
 * State            : $State$
 */
#ifndef __IMAGE_MEMORY_H_INCLUDED__
#define __IMAGE_MEMORY_H_INCLUDED__

/**
* Abstract Base class for image memory manager.
*/
class CImageMemory {
public:

	/**
	* Default constructor for creating new CImageMemory object
	*/
	CImageMemory() {}

	/**
	* Destructor for destroying CImageMemory object
	*/
	virtual ~CImageMemory() {}

	/**
	* Allocates memory for a raw image
	*
	* @param length   Number of bytes to be allocated
	* @param index    Image (frame) index within the DICOM object
	*
	* @return a pointer to the allocated memory block or NULL
	*/
	virtual char * allocate(unsigned int length, int index) = 0;

	/**
	* Releases memory of a raw image
	*
	* @param block    Memory block to be released
	* @param length   Number of allocated bytes in the block
	* @param index    Image (frame) index within the DICOM object
	*/
	virtual void release(char * block, unsigned int length, int index) = 0;

	/**
	* Chunks allocated memory block of a raw image.
	* New length should be less or equal than the original length.
	* The chunked memory block might differ from the original one.
	*
	* @param block       Memory block to be chunked
	* @param oldLength   Number of allocated bytes in the block
	* @param newLength   Number of bytes used in the block (chunked length)
	* @param index       Image (frame) index within the DICOM object
	*
	* @return a pointer to chunked memory block or NULL
	*/
	virtual char * chunk(char * block, unsigned int oldLength, unsigned int newLength, int index) = 0;
};

#endif // __IMAGE_MEMORY_H_INCLUDED__
