/*
 * Template class to auto-delete objects
 */

#ifndef IMUTILS_SRC_AUTODELETE_H_
#define IMUTILS_SRC_AUTODELETE_H_

#include <stdlib.h>

namespace im {

/**
 * Template class to auto-delete objects
 *
 * Usage:
 *
 * {
 *   AutoDelete<Class> autoDelete;
 *   Class * c = new Class();
 *   autoDelete.set( c );
 *   ...
 * }
 *
 * or
 *
 * {
 *    AutoDelete<Class> autoDelete( new Class() );
 *    Class * c = autoDelete.get();
 *    ...
 * }
 *
 */
template< class T >
class AutoDelete {
		T * m_object;

	public:
		AutoDelete() :
				m_object( NULL ) {
		}

		AutoDelete( T * object ) :
				m_object( object ) {
		}

		~AutoDelete() {
			if ( m_object != NULL ) {
				delete m_object;
				m_object = NULL;
			}
		}

		inline void set( T * object ) {
			m_object = object;
		}

		inline T * get() {
			return m_object;
		}
};

}

#endif /* IMUTILS_SRC_AUTODELETE_H_ */
