/*
 * Check Sum using the same algorithm as the viewer and Java code:
 *  - CreateChecksum() in pbuilder.cpp (csz)
 *  - imagemed.layout.macro.input.TextInputMacro.createChecksum(String) (rty)
 *
 */
#ifndef SRC_CHECKSUM_H_
#define SRC_CHECKSUM_H_

namespace im {

/**
 * Class to generate a check sum for a particular text
 *
 * Usage:
 *
 * {
 *    CheckSum cs( originalText );
 *    ...
 *    cs.getCheckSum();
 *    ...
 * }
 */
class CheckSum {
		char * checkSum;

	public:

		/**
		 * Constructs a new check sum object to generate check sum for the particular text.
		 *
		 * @param text    generate the CheckSum for this text
		 * @param length  the length of the check sum to generate (default: 10)
		 */
		CheckSum( const char * text, size_t length = 10 );

		/**
		 * Destroys this CheckSum object
		 */
		~CheckSum();

		/**
		 * Returns the generated check sum.
		 * Note that the returned value will become invalid when
		 * this CheckSum object get destroyed.
		 *
		 * @return the check sum
		 */
		inline const char * getCheckSum() {
			return checkSum;
		}
};

/**
 * Generates a check sum for the particular text.
 *
 * @param text    generate check sum for this text
 * @param length  the length of the check sum to generate (default: 10)
 *
 * @return the check sum (the caller shall release)
 */
char * genCheckSum( const char * text, size_t length = 10 );

}

#endif /* SRC_CHECKSUM_H_ */
