/*
 * RunCommand.h
 *
 */

#ifndef IMUTILS_INCLUDE_IMUTILS_RUNCOMMAND_H_
#define IMUTILS_INCLUDE_IMUTILS_RUNCOMMAND_H_

#include <exception>
#include <string>
#include <list>
// #include <array>
#include <initializer_list>

namespace erad { namespace testing {

class CommandResult;

class CmdArgArray {
	protected:
		CmdArgArray( size_t size );

	public:
		~CmdArgArray();

		// disable specials
		CmdArgArray() = delete;
		CmdArgArray( const CmdArgArray & orig );
		CmdArgArray( CmdArgArray && orig);

		CmdArgArray& operator= (const CmdArgArray & orig) = delete;
		CmdArgArray& operator= (CmdArgArray && orig) = delete;


		void unset( size_t ndx );

		bool set( size_t ndx, const char * value );

		inline char * const * getArray() const {
			return theArray;
		}

		inline const char * getFirst() const {
			return theArray[ 0 ];
		}

		CommandResult run() const;

		friend std::ostream& operator<< ( std::ostream& out, const CmdArgArray& array );

		static CmdArgArray initCmdArray( const std::list<std::string> & command );

		static CmdArgArray initCmdArray( const std::initializer_list<std::string> & command );

	protected:
		template<typename C>
		static CmdArgArray toCmdArray( C && command );


	private:

		size_t size;
		char ** theArray;
};


class CommandException : public std::exception {

	public:
		CommandException( const std::string & msg );
		virtual ~CommandException();

		// disable specials
		CommandException() = delete;
		CommandException( const CommandException & orig ) = delete;
		CommandException( CommandException && orig) = default;

		CommandException& operator= (const CommandException & orig) = delete;
		CommandException& operator= (CommandException && orig) = delete;

		virtual const char* what() const noexcept;

	private:
		std::string msg;
};



class CommandResult {
public:
	CommandResult() = delete;
	CommandResult( const CommandResult & orig ) = delete;
	CommandResult( CommandResult && orig) = default;
	~CommandResult();

	CommandResult& operator= (const CommandResult & orig) = delete;
	CommandResult& operator= (CommandResult && orig) = default;

	inline const std::string & getOutput() const {
		return output;
	}

	inline const std::string & getError() const {
		return error;
	}

	inline int getExitCode() const {
		return exitCode;
	}

protected:
	CommandResult( int exitCode, std::ostringstream & outputStream, std::ostringstream & errorStream );

	static CommandResult runCommand( const char * file, char* const args[] );

private:
	int exitCode;
	std::string output;
	std::string error;
	friend class CmdArgArray;
};



} }

#endif /* IMUTILS_INCLUDE_IMUTILS_RUNCOMMAND_H_ */
