/*
 * SessionDB.h
 *
 *  Created on: Mar 28, 2022
 *      Author: joco
 */

#ifndef IMUTILS_INCLUDE_IMUTILS_SESSIONDB_H_
#define IMUTILS_INCLUDE_IMUTILS_SESSIONDB_H_

#include <string>
#include <list>
#include "imutils/DBConnector.h"

namespace erad::db {


/**
 * Interface to manage reference counters
 */
class SessionDB {

	public:
		virtual ~SessionDB() {
		}

		/**
		 * Checks if a sessionID is valid for a given IP address
		 *
		 * @param id Session ID
		 * @param ip IP address
		 *
		 * @throw DBException if an DB error occurs
		 *
		 * @return true if the session is valid, false otherwise
		 */
		virtual bool checkSession( const std::string& id, const std::string& ip, int& priority /*out*/, bool& lossy /*out*/) = 0;

};


/**
 * Reference counter implementation managing reference counters in MySQL database tables.
 */
class SessionDBSql : public SessionDB, public PSCache {

	public:

		/**
		 * Constructs a new SessionDBSql object to prepare and execute MySQL queries
		 *
		 * @param pDB connector to the MySQL database
		 */
		SessionDBSql( DBConnectorSql& pDB );
		virtual ~SessionDBSql();

		/**
		 * @copydoc SessionDB::checkSession(const std::string&,const std::string&)
		 */
		virtual bool checkSession( const std::string& id, const std::string& ip, int& priority /*out*/, bool& lossy /*out*/ ) noexcept ( false )
				override;

	private:

		std::string sessionTbl;

		PsQuery checkSessionQuery; // NEW_LOCK( () -> "insert into " + TableEnum.EPLOCKTBL.getTableSQLName() + " (LCKID,LCKRESOURCE,LCKACTOR,LCKTYPE,LCKTIME,LCKCOUNTER) values (?,?,?,?,NOW(),1)" )
};

} // namespace

#endif /* IMUTILS_INCLUDE_IMUTILS_SESSIONDB_H_ */
