#ifndef __SYSTEMCONFIG_H__
#define __SYSTEMCONFIG_H__

#include <string>
#include <vector>


enum ModuleType { MT_TASKD, MT_TABLEBROKER, MT_USERMANAGER };


class HostConfig {
	std::string m_name;
	int m_webSocketPort;
	std::vector<ModuleType> m_modules;
public:
	HostConfig(const std::string& name, int webSocketPort, const std::vector<ModuleType> modules);
	const std::string& getName() const;
	int getWebSocketPort() const;
	const std::vector<ModuleType>& getModules() const;
};


class MachineConfig {
	std::string m_id;
	std::string m_hostname;
	int m_rmiRegistryPort;
	std::vector<HostConfig> m_hosts;
public:
	MachineConfig(const std::string& id, const std::string& hostname, int rmiRegistryPort, const std::vector<HostConfig> hosts);
	const std::string& getId() const;
	const std::string& getHostname() const;
	int getRmiRegistryPort() const;
	const std::vector<HostConfig>& getHosts() const;
};


class SystemConfig {
public:
	static const char* s_szModulesXml;
private:
	std::vector<MachineConfig> m_machines;
public:
	SystemConfig(const std::string& modulesXml);
	const std::vector<MachineConfig>& getMachines() const;
};


#endif // __SYSTEMCONFIG_H__
