#pragma once
#include "imutils/GCPConnector.h"
#include "imutils/IMProperties.h"
#include <curl/curl.h>
#include <dcmtk/ofstd/ofstring.h>
#include <google/cloud/credentials.h>
#include <nlohmann/json.hpp>

namespace erad::gcp {

class TransactionServiceConnector : public GCPConnector {
	public:
		/**
		 * Constructor for initialization GCP Network for transaction service
		 *
		 */
		TransactionServiceConnector();

		/**
		 * Set up curl as part of network request initialization.
		 *
		 * @param accessToken   access token
		 *
		 * @return true if success
		 */
		virtual OFBool setupCurlForCloudServiceType( const OFString& accessToken ) override;

		/**
		 * Get the  Query-Retrieve service url.
		 *
		 * @return Query-Retrieve service url
		 */
		OFString getQueryBaseURL();

		/**
		 * Function to perform the cURL request and return the result.
		 *
		 * @param sopInstanceUid sop instance uid
		 * @param readBuffer - response buffer data
		 * @param response_code - response code
		 *
		 * @return the curl code
		 */
		CURLcode performCurlRequest( const std::string& sopInstanceUid, std::string& readBuffer, long& response_code );

		/**
		 * Perform the query to the query-service url
		 *
		 * @param url - query url
		 * @param responseData - response data
		 *
		 * @return the status
		 */
		OFBool performQuery( const std::string& url, std::string& responseData );

		/**
		 *Destructor for destroying the TransactionServiceConnector object
		 */
		~TransactionServiceConnector();

	private:
		/**
		 * function to set config path
		 *
		 * @throws IMException if an error occures setting config path
		 *
		 * @return device config path
		 */
		 OFString getConfigPath(); 

		/** Query service base url*/
		OFString m_baseURL;

		/**
		 * Handle the curl response.
		 *
		 * @param res   curl response code
		 *
		 * @return true if success
		 */
		OFBool handleCurlResponse( CURLcode res );

};

} // namespace erad::gcp
