#ifndef ERAD_IMUTILS_CLASSES_COMPACTDCMDUMP_H_
#define ERAD_IMUTILS_CLASSES_COMPACTDCMDUMP_H_

#include <iostream>
#include <sstream>

#include <dcmtk/dcmdata/dcitem.h>

namespace erad { namespace utils {

/**
 * Generates a compact dump from a DICOM dataset
 * by removing unnecessary white spaces and comments from dcmdump
 */
class CompactDcmDump {
	private:
		/** the original dcmdump */
		std::stringstream dcmDump;
		/** true indicates to include VRs in the dump */
		bool includeVR;

		inline void skipWhiteSpaces( const char*& p ) const {
			while ( isspace( *p ) ) {
				p++;
			}
		}

		inline void skipNonWhiteSpaces( const char*& p ) const {
			while ( !isspace( *p ) && *p != '\0') {
				p++;
			}
		}

		inline void printNonWhiteSpaces( const char*& p, std::ostream& out ) const {
			while ( !isspace( *p ) && *p != '\0' ) {
				out << *(p++);
			}
		}

	protected:

		void printTo( std::ostream& os );

	public:
		/**
		 * Constructs a CompactDcmDump object and initializes
		 * with the dump of the provided DICOM item
		 *
		 * @param dcmItem the DICOM item to dump
		 * @param includeVR controls whether or not VRs to be included to the dump (defult: true)
		 */
		CompactDcmDump( DcmItem& dcmItem, bool includeVR = true );
		~CompactDcmDump();

		friend std::ostream& operator<<( std::ostream& out, CompactDcmDump& compactDump ) {
			compactDump.printTo( out );
			return out;
		}
};

} } // namespace

#endif /*  ERAD_IMUTILS_CLASSES_COMPACTDCMDUMP_H_ */
