/*
 * Component: utils
 *
 * Description:
 *    Hedaer file for helper functions for programs using DCMTK
 *
 * Last Updated     : $Author$
 * Update Date Time : $Date$
 * RCS File Name    : $RCSfile$
 * RCS Revision     : $Revision$
 * Locked by        : $Locker$
 * Symbolic name    : $Name$
 * State            : $State$
 */
#ifndef __DCM_UTILS_H_INCLUDED__
#define __DCM_UTILS_H_INCLUDED__

#include <dcmtk/dcmdata/dctk.h>

class DcmSeeker {
private:
	DcmItem & m_item;
	DcmObject * m_current;
	DcmTagKey m_lastKey;

public:
	DcmSeeker(DcmItem & item);

	DcmObject * find(const DcmTagKey & key);
	void removeCurrent();

	void reset();
};

OFString replaceInOFString(const OFString &srcstr, const OFString &pattern, const OFString &substitute);

void errmsg(const char * appName, const char *msg,...);

void errmsg(const char *msg,...);

char * createSOPInstanceUID(char *prefix);

char * createSOPInstanceUID();

void moveElement(DcmElement *elem, DcmItem * src, DcmItem * dst);

void copyItem(DcmItem * item, DcmSequenceOfItems * seq);

void passSequenceElement(DcmSequenceOfItems *seq, DcmItem *outItem);

void copyElementToItem(DcmElement *elem, DcmItem * container, OFBool replaceOld = OFFalse);

void passSequence(DcmTagKey key, DcmItem *inds, DcmItem *outds);

void passOrCreateElem(DcmElement *elem, DcmTagKey key, DcmItem *outds);

void passOrCreateTag(DcmTagKey key, DcmItem *inds, DcmItem *outds);

void passTag(DcmTagKey key, DcmItem *inds, DcmItem *outds);

DcmSequenceOfItems * createReferenceSequence(DcmTagKey sequence, const char* clUID, char* instUID);

DcmElement* getElement(DcmItem* ds, DcmTagKey key);

void getValue(DcmItem* ds, DcmTagKey key, char *&value);

DcmFileFormat * readExistingDicomFile(const char * fname);

DcmFileFormat * readOrCreateDicomFile(const char * fname);

OFBool writeDicomFile(const char * ofname, DcmFileFormat * dcmff);

DcmElement * copyElement(DcmElement * elem);

// OBSOLETE
bool OBS_startTask(const OFString & cmd, const OFString & priority, const OFString & type, const OFString & queue);

OFCondition applyModifications(DcmDataset & imageDataset, DcmDataset & pbrDataset, OFList<DcmTagKey> * modList);

OFCondition applyModifications(DcmDataset & imageDataset, DcmDataset & pbrDataset);

OFCondition applyStudyModification(DcmDataset & imageDataset,
		DcmItem & studyItem, DcmSequenceOfItems *& seriesSeq, OFList<DcmTagKey> * modList, OFBool ignoreModality);

OFCondition applySeriesModification(DcmDataset & imageDataset,
		DcmSequenceOfItems & seriesSeq, DcmSequenceOfItems *& objectSeq, OFList<DcmTagKey> * modList, OFBool ignoreModality);

OFCondition applyInstanceModification(DcmDataset & imageDataset,
		DcmSequenceOfItems & instanceSeq, OFList<DcmTagKey> * modList, OFBool ignoreModality);


#endif // __DCM_UTILS_H_INCLUDED__
