#!/bin/bash
# Component: dotcom
#
# Description:
#   Sync all study.
#
# Usage:
#    changeId.sh -o [oldid] -n [newid]  
#

. $HOME/etc/setenv.sh

OLD_ID=""
NEW_ID=""

while getopts "o:n:" Option ; do
	case $Option in
		o) OLD_ID=$OPTARG;;
		n) NEW_ID=$OPTARG;;
	esac
done

if [ -z "$OLD_ID" -o -z "$NEW_ID" ] ; then
	echo Usage:
	echo   changeId.sh -o [oldid] -n [newid]
	echo
	exit 1;
fi

echo "Are you sure to change $OLD_ID to $NEW_ID? (y/n)"
read -n 1 ANSWER
if [ $ANSWER != "y" -a $ANSWER != "Y" ] ; then
	exit 0;
fi

echo "1. database ..."
${TOOLS_HOME}/sql.sh "update Target set ID='$NEW_ID' where ID='$OLD_ID'"
if [ $? -gt 0 ] ; then
	echo "Mysql not running, can not modify Target table!"
	exit 0;
fi
echo "2. storestates ..."

for study in `$REPOSITORYHANDLER_HOME/bin/listDirectories $REPOSITORY_DATA_DICOM` ; do
	location=`$REPOSITORYHANDLER_HOME/bin/locateResource $REPOSITORY_DATA_DICOM $study`
	parent=`echo "$location" | sed "s/$study$//g"`
	grep -H "$OLD_ID" $location/.info/storestate.rec | sed "s:$parent::g"  | \
		tr "[:/=\"]" " " | tr -s " " | cut -d" " -f1,4,5 | while read LINE; do
		STUDY=`cut -d" " -f1 <<<$LINE`;
		FIELD=`cut -d" " -f2 <<<$LINE`;
		VALUE=`cut -d" " -f3 <<<$LINE`;
		NEWVALUE=`sed "s:\<$OLD_ID\>:$NEW_ID:g" <<<$VALUE`;

		if [ $FIELD == "OwnerDevice" ] ; then
			echo "  $FIELD  $STUDY"
			$UTILS_HOME/bin/storestate --study=$STUDY --setowner=$NEW_ID
			$UTILS_HOME/bin/storestate --study=$STUDY --removedevice=$OLD_ID
		elif [ $FIELD == "StagedDevices" ] ; then
			echo "  $FIELD  $STUDY"
			$UTILS_HOME/bin/storestate --study=$STUDY --setstaged=$NEW_ID
			$UTILS_HOME/bin/storestate --study=$STUDY --removedevice=$OLD_ID	
		elif [ $FIELD == "ArchiveDevice" ] ; then
			echo "  $FIELD  $STUDY"
			$UTILS_HOME/bin/storestate --study=$STUDY --archive=$NEWVALUE
		elif [ $FIELD == "SourceDevice" ] ; then
			echo "  $FIELD  $STUDY"
			$UTILS_HOME/bin/storestate --study=$STUDY --source=$NEWVALUE
		fi
	done;
done;

echo "3. references ..."


grep -r $OLD_ID $REFCOUNT_VAR | sed "s:$REFCOUNT_VAR/::g" | tr "/:=" " " | 
		awk -vOld_id=$OLD_ID -vNew_id=$NEW_ID -vUtilsHome="$UTILS_HOME" '
		{ 
			Type = $1
			if(Type != "OBJECTFILE") {
				Object=$(NF-2);
			}
			else {
				Object=$(NF-3)"/"$(NF-2);
			}

			OldRef=$(NF);
			tmp = $(NF);
			gsub("_" Old_id "_", "_" New_id "_", tmp);
			gsub("_" Old_id "\\>", "_" New_id, tmp);
			gsub("\\<" Old_id "\\>", New_id, tmp);
			gsub("\\<" Old_id "_", New_id "_", tmp);
			NewRef=tmp;
			printf("%s/bin/refcount -a %s -i %s -t %s\n",UtilsHome, NewRef, Object, Type);
			printf("%s/bin/refcount -r %s -i %s -t %s\n",UtilsHome, OldRef, Object, Type);
		}' |/bin/sh

echo OK

