#!/bin/gawk -f
#
# Description:
#   Figures out data for Archive table from dcmdump of object files
#
# stdin is expected to be dcmdump of DICOM objects
# output is field values in
# <database name>="<field value>"
#
#  quotes and double quotes and backslashes in <filed value> are escaped
#
# Last Updated     : $Author $
# Update Date Time : $Date $
# RCS File Name    : $RCSfile $
# RCS Revision     : $Revision $
# Locked by        : $Locker $
# Symbolic name    : $Name $
# State            : $State $
#

function getValue(str) {
	p1 = index(str,"[");
	p2 = index(str,"]");

	res = "";
	if (p1 > 0 && p2 > 0) {
		res = substr(str, p1+1 , p2-p1-1);
		gsub(bs bs, bs bs bs bs, res);
		gsub(quote, bs quote, res);
		gsub(dquote, bs dquote, res);
		gsub(btick, bs btick, res);
	}
	return res;
}

function printValue(name, str) {
	val = getValue(str);
	if (val != "")
		print name "=\"" val "\"";
}

BEGIN {
	quote = "'";
	dquote = "\"";
	btick = "`";
	bs = "\\";
}

($1 == "(0008,0201)") { printValue("TZOFFSET",$0); next;}
($1 == "(0010,0010)") { printValue("PNAME",$0); next;}
($1 == "(0010,0020)") { printValue("PID",$0); next;}
($1 == "(0010,0030)") { printValue("PBDATE",$0); next;}
($1 == "(0010,0032)") { printValue("PBTIME",$0); next;}
($1 == "(0008,0050)") { printValue("ACCNO",$0); next;}
($1 == "(0008,0060)") { printValue("MODALITY",$0); next;}
($1 == "(0018,0015)") { printValue("BODYPART",$0); next;}
($1 == "(0008,1030)") { printValue("DESCRIPTION",$0); next;}

($1 == "(0008,0020)") { thedate = getValue($0); next;}
($1 == "(0008,0030)") { thetime = getValue($0); next;}
($1 == "(0008,0012)") { icrdate = getValue($0); next;}
($1 == "(0008,0013)") { icrtime = getValue($0); next;}
($1 == "(0008,0022)") { acqdate = getValue($0); next;}
($1 == "(0008,0032)") { acqtime = getValue($0); next;}
($1 == "(0008,0023)") { imgdate = getValue($0); next;}
($1 == "(0008,0033)") { imgtime = getValue($0); next;}

END {
	if (thedate == "") {
		thedate = icrdate;
		thetime=icrtime;
	}

	if (thedate == "") {
		thedate = acqdate;
		thetime=acqtime;
	}

	if (thedate == "") {
		thedate = imgdate;
		thetime=imgtime;
	}

	if (thedate != "") {
		print "THEDATE=\"" thedate "\"";
		print "THETIME=\"" thetime "\"";
	}
}
