#!/bin/bash
#
# Component: tools
#
# Description:
#  Collects information from storestate.rec files
#
# Usage:
#		getstates.sh [-o outputdir] [list of studyIUIDs]
#			where -o outputdir must be the first and the second parameter if defined
#			(default output directory is $TMP/getstates)
#
# getstates will generate output files in $TMP/getstates if no 
# outputdir is specified.
#
#  self.ls      list of studies on current server
#  child.ID.ls  list of expected studies on child server(s)
#  error.log    errors detected during execution
#
# The format of *.ls output files:
#
#   <Study Instance UID>  <flag>
#
#  Flag can be:
#     O - the study is on the owner branch
#     S - the study is on the staged branch
#
#
#  Last Modified by : $Author$
#  Last Modified at : $Date$
#
#  RCS File Name    : $RCSfile$
#  RCS Revision     : $Revision$
#  Locked by        : $Locker$
#  Symbolic name    : $Name$
#  State            : $State$
#

. $HOME/etc/setenv.sh

if [ -n "$1" -a "$1" = "-o" ]; then
  TDIR=$2;
  shift 2
else
  TDIR=$TMP/getstates
fi

if [ -n "$1" -a "$1" = "-h" ]; then
  REFERER=$2;
  shift 2
else
  REFERER=$TMP/getstates
fi

[ ! -d $TDIR ] && mkdir $TDIR
rm $TDIR/* 2>/dev/null

echo "select ID from $TARGETTBL where TYPE='C' or TYPE='P';" | $MYSQL_HOME/bin/mysql -N imagemedical | while read i; do
	>$TDIR/child.$i.l
done

>$TDIR/self.l

LS=`$REPOSITORYHANDLER_HOME/bin/listDirectories $REPOSITORY_DATA_DICOM`

if [ " " != " $1" ] ; then
	ORIGLS=$LS
	FILTER=$*
	LS=""
	for i in $ORIGLS ; do
		for j in $FILTER ; do
			if [ $i == $j ] ; then
				LS="$LS $i"
			fi
		done
	done
fi

for s in $LS ; do
	echo Checking study $s
	studydir=`$REPOSITORYHANDLER_HOME/bin/locateResource $REPOSITORY_DATA_DICOM $s`
	cd $studydir
	cd ..

	if [ ! -f $s/.info/storestate.rec ] ; then
		echo "$s has no storestate.rec (junk directory in $REPOSITORY_DICOM_DATA?)" >> $TDIR/error.log
	else

		# read storestate.rec file

		StudyOwner=""
		StagedDevices=""
		OwnerDevice=""

		. $s/.info/storestate.rec

		if [ $? != 0 ] ; then
			echo "$s has corrupt storestate.rec file!" >>$TDIR/error.log
		fi

# 		echo Before:
# 		echo "OwnerDevice: $OwnerDevice"
# 		echo "StagedDevices: #$StagedDevices#"
# 		echo "StudyOwner: $StudyOwner"

		# NULL is obsolete?

		if [ "$OwnerDevice" = "NULL" ] ; then
			OwnerDevice=""
		fi

		# parent list

		if [ "$REFERER" ]; then			
			if [ -n "$StudyOwner" ]; then
				echo "$s O" >> $TDIR/self.l
			elif [ -n "$OwnerDevice" ]; then
				echo "$s S" >> $TDIR/self.l
			elif [ "$OwnerDevice" == "$REFERER" ]; then
				echo "$s S" >> $TDIR/self.l
			elif [ -n "$OwnerDevice" && "$OwnerDevice" != "$REFERER" ]; then
				echo "$s O" >> $TDIR/self.l
			fi
		else				
			if [ -n "$StudyOwner" -o -n "$OwnerDevice" ] ; then
				echo "$s O"  >> $TDIR/self.l
			else
				echo "$s S"  >> $TDIR/self.l
			fi
		fi

		# child lists

		if [ -n "$StagedDevices" ] ; then
			#check Staged IPs for consistency

#			echo Staged IPs: $StagedIPs
			for i in $StagedDevices; do echo $i; done | sort > $TDIR/st
			sort -u $TDIR/st > $TDIR/st2
			if diff $TDIR/st $TDIR/st2 ; then
				:
			else
				echo "$s has multiplicity in  staged Devices!" >>$TDIR/error.log
				# fix it
				StagedDevices=`cat $TDIR/st2 | while read i; do echo -n $i; done`
			fi
			rm $TDIR/st
			rm $TDIR/st2

			st=""
			for i in $StagedDevices; do
				if [ ! -f $TDIR/child.$i.l ] ; then
					echo "$s has StagedDevices reference to non-child $i"  >>$TDIR/error.log
				else
					echo "$s S"  >> $TDIR/child.$i.l
					st="${st}${i} ";
				fi
			done
			StagedDevices=${st% }
		fi

		if [ -n "$OwnerDevice" ] ; then
			if [ ! -f $TDIR/child.$OwnerDevice.l ] ; then
				echo "$s has OwnerDevice reference to non-child $i"  >>$TDIR/error.log
				#fix it, remove OwnerDevice...
				OwnerDevice=""
				StudyOwner="yes"
			else
				echo "$s O"  >> $TDIR/child.$OwnerDevice.l
			fi
		fi

#		echo After:
#		echo "OwnerDevice: $OwnerDevice"
#		echo "StagedDevices: #$StagedDevices#"
#		echo "StudyOwner: $StudyOwner"

	fi

#	break;
done

for i in $TDIR/*.l; do
	sort ${i} > ${i}s
	rm ${i}
done
