#!/bin/bash
#
# Component: tools
#
# Purpose:
#   Starts sending DICOM objects to the archive
#
# Usage:
#      initarchive.sh
#
# Description:
#
# Starts sending all studies/DICOM objects stored in the database (Work List) to the archive.
#
# initarchive.sh will schedule $ARCHIVE_HOME/put.sh tasks.
#
# initarchive.sh can be used to send existing DICOM objects from the Work List to the newly configured archive.
#
#
# Last Updated     : $Author$
# Update Date Time : $Date$
# RCS File Name    : $RCSfile$
# RCS Revision     : $Revision$
# Locked by        : $Locker$
# Symbolic name    : $Name$
# State            : $State$
#

. $HOME/etc/setenv.sh

Archive="no"

[ -f "$ARCHIVE_VAR/archive.cfg" ] && . $ARCHIVE_VAR/archive.cfg

if [ "$Archive" != "yes" ] ; then
	echo "No archive has been set." >&2
	exit 1
fi

SELF_SERVER_ID=""
[ -f "$CONF/serverid.cfg" ] && . $CONF/serverid.cfg

if [ -z "$SELF_SERVER_ID" ] ; then
	echo "Server ID could not be recognized." >&2
	exit 1
fi

parentId=`echo "select ID from $TARGETTBL where TYPE='P';" | $MYSQL_HOME/bin/mysql -N $DBNAME`
if [ $? -ne 0 ] ; then
	echo "MySQL error" >&2
	exit 1
fi

# iterate through the studies
Pid=$$
DATE=`date +%Y%m%d%H%M%S`


echo "select STYIUID, FNAME from $OBJECTTBL order by STYIUID;" | \
$MYSQL_HOME/bin/mysql -N $DBNAME 2>/dev/null | \
while read Study File ; do

	ArchiveDevice=""
	. $CASES_HOME/includeStorestate.shi "$Study"
	if [ $? -ne 0 ] ; then 
		echo "Could not read storestate.rec file of $Study" >&2
		continue
	fi

	if [ -z "$ArchiveDevice" ] ; then
		# Set ArchiveDevice for the study
		$UTILS_HOME/bin/storestate -S "$Study" --archive="@Archive-$SELF_SERVER_ID"
		if [ $? -ne 0 ] ; then
			echo "Could not set ArchiveDevice for $Study" >&2
			continue
		fi
	else
		archDev="${ArchiveDevice#@Archive-}"
		if [ "$archDev" = "@TOP" ] ; then
			if [ -n "$parentId" ] ; then
				echo "Legacy archive for $Study $File is parent server ($parentId)" >&2
				continue
			fi
		elif [ "$archDev" != "$SELF_SERVER_ID" ] ; then
			echo "$Study $File has already been archived at $archDev"
			continue;
		fi
	fi

	
	# add 'archive' reference to the OBJECTFILE. It will be removed by archive/put.sh
	$UTILS_HOME/bin/refcount -E -a archive -i "${Study}/${File}" -t "OBJECTFILE" >>$LOG/refcount.error 2>&1

	if [ $? -ne 0 ] ; then
		echo "Could not add 'archive' reference to file ${Study}/${File}" 1>&2
		continue
	fi

	DeltaPriority=""
	[ "${file#PbR.}" != "$file" ] && DeltaPriority="DELTA_ARCHIVE_PUT_PBR"

	# start put the study to the archive
	$TASKD_HOME/createtaskfileapp -p "ARCHIVE_PUT" -t "sh" -d "$DeltaPriority" -c "$ARCHIVE_HOME/put.sh -s \"$Study\" -f \"$File\""

done
