#!/bin/bash
#
# Starts sending all studies/DICOM objects stored in the database to the parent dot-com server.
#

. $HOME/etc/setenv.sh

# Relative priority to start study synchronizer and forward tasks
REL_PRIORITY="3000"

[ -f "$CONF/serverid.cfg" ] && . "$CONF/serverid.cfg"
JOB_ID="$SELF_SERVER_ID-INIT$(date +%s)"

PARENT=`echo "select ID from $TARGETTBL where TYPE='P';" | \
		$MYSQL_HOME/bin/mysql -N $DBNAME 2>/dev/null`

if [ -z "$PARENT" ] ; then
	echo "No parent server has been set."
	exit 1
fi

isDbOnlyParent="false"

dbOnlyDevices=`$DOTCOM_HOME/getDatabaseDevice.awk $CONF/dotcom.jcfg`
if [ -n "$dbOnlyDevices" ] ; then
	for dev in $dbOnlyDevices; do
		[ "$dev" = "$PARENT" ] && isDbOnlyParent="true"
	done
fi

# iterate through the studies
echo "select STYIUID from $STUDYTBL;" | $MYSQL_HOME/bin/mysql -N $DBNAME 2>/dev/null | \
while read study ; do
	metaDir=$( $UTILS_HOME/bin/meta -s "${study}" )

	# add reference to STUDYDB to make sure the study will not be deleted
	$UTILS_HOME/bin/refcount -E -t "STUDYDB" -i "$study" -a "initdotcom" >>$LOG/refcount.error 2>&1
	if [ $? -ne 0 ] ; then
		echo "Could not add reference to study $study" >&2
		continue
	fi

	# check if the storestate.rec file exists
	ProcessMode="Process"
	OwnerDevice="NULL"

	storestateParams="--processmode=Process --owner=yes"
	if [ -f "${metaDir}/storestate.rec" ] ; then
		# check if the storestate.rec file is correct
		. $CASES_HOME/includeStorestate.shi "$study"

		if [ "$OwnerDevice" != "NULL" ] ; then
			OWNER=`echo "select ID from $TARGETTBL where ID='$OwnerDevice' and TYPE='C' or TYPE='P';" |\
					$MYSQL_HOME/bin/mysql -N $DBNAME 2>/dev/null`
			if [ -n "$OWNER" ] ; then
				storestateParams=""
			else
				storestateParams="$storestateParams --removedevice \"$OwnerDevice\""
			fi
		fi
	fi

	if [ -n "$storestateParams" ] ; then
		$UTILS_HOME/bin/storestate --study "$study" $storestateParams
	fi

	if [ "$isDbOnlyParent" = "true" ] ; then
		# start synchronizing the database to the parent
		$TASKD_HOME/createtaskfileapp -p "STUDY_SYNCHRONIZER" -j "$JOB_ID" -r "${REL_PRIORITY}" -t "class" -c "dotcom.StudySynchronizerTask -f -s \"$study\""
	else
		# start forwarding the study to the parent
		$TASKD_HOME/createtaskfileapp -p "AUTO_FORWARD" -j "$JOB_ID" -r "${REL_PRIORITY}" -t "class" -c "cases.Forward -H $PARENT -u system -s \"$study\""
	fi

	# remove reference from STUDYDB
	$UTILS_HOME/bin/refcount -t "STUDYDB" -i "$study" -r "initdotcom" >>$LOG/refcount.error 2>&1
done
