#!/bin/bash
#
# $1 is a filename which contains the result of an ls
# executed on child in data/dicom directory
#
# $2 device id of the child server
#
# restorestorestates.sh will create a storestate.rec file for the studies
# listed in the ls file and missing from data dicom as Deleted studies
#

. $HOME/etc/setenv.sh

if [ -z "$2" ] ; then
	echo "missing argument"
	echo "usage: restorestorestates.sh <lsfile> <childID>"
	exit 1
fi

lsfile=$1
childID=$2

RES_CHILDID=`echo "select ID from $TARGETTBL where ID='$childID' and TYPE='C';" | \
		$MYSQL_HOME/bin/mysql -N DBNAME`

if [ "$childID" != "$RES_CHILDID" ] ; then
	echo "$childID doesn't exists in dicom devices table as a child server"
	exit 1
fi

CWD=`pwd`

for study in `cat $lsfile` ; do
	studydir=`$REPOSITORYHANDLER_HOME/bin/locateResource $REPOSITORY_DATA_DICOM $study`
	if [ $? -ne 0 ] ; then
		RES_STUDY=`echo "select STYIUID from $ARCHIVETBL where STYIUID='$study';" | \
		$MYSQL_HOME/bin/mysql -N DBNAME`

		if [ "$RES_STUDY" = "$study" ] ; then
			$REPOSITORYHANDLER_HOME/bin/createDirectory $REPOSITORY_DATA_DICOM $study
			$UTILS_HOME/bin/storestate -C --study "$study" --processmode=Deleted --setowner "$childID"

			$UTILS_HOME/bin/refcount -n "child_$childID" -t STUDYDIR -i "$study" >>$LOG/refcount.error 2>&1
		fi
	fi
done

cd $CWD
