#!/bin/bash
#
# Component: tools
#
# Description:
#   Creates usage/monitoring graphs
#
# Usage:
#   See usage_ or run makegraph0.sh -h
#

usage_()
{
	Quot=$"'"
	cat <<-END_OF_USAGE >&2
		Usage:

		makegraph0.sh [<options>]

		Options:
		    -d <date>   : date YYYY-MM-DD (default: today)
		    -p <post>   : info.log postprocessor
		    -f <graph>  : name of the .bmp file
		    -c          : create csv excel dump
		    -r          : reload (don${Quot}t use cached graph)

	END_OF_USAGE
} # usage_


DIR=$( dirname $0 )
if [ -d ~/htdocs/tmp ] ; then 
  if [ ! -L ~/htdocs/tmp ] ; then
    # old style tmp physically under htdocs -- move to tmp/htdocs
    mv ~/htdocs/tmp ~/tmp/htdocs
  fi
fi

[ ! -d ~/tmp/htdocs ] && mkdir ~/tmp/htdocs 2>/dev/null
[ ! -L ~/htdocs/tmp ] && ln -sT ~/tmp/htdocs ~/htdocs/tmp 2>/dev/null

Post="";
Date="";
Bmp="";
isCSV="";
POSTPROC=cat;
FILE0=monitor.log

while getopts 'd:p:f:cr' OPT; do
	case $OPT in
	d) Date="$OPTARG" ;;
	p) Post="$OPTARG" ;;
	f) Bmp="$OPTARG" ;;
	c) isCSV="true" ;;
	r) doReload="true" ;;
	*) usage_; exit 1 ;;
	esac
done
shift $((OPTIND - 1))

[ -z "$Bmp" ] && Bmp=$1;

if [ -n "$Post" ] ; then
   if [ ! -f $DIR/post_$Post.sh ] ; then
     echo No postprocessor for $Post
     exit 1;
   fi
   POSTPROC=$DIR/post_$Post.sh ;
   FILE0=`$POSTPROC -f`
fi

Today=`date +%F`;
[ -z "$Date" ] && Date=$Today;

[ -n "$Date" ] && Bmp=$Bmp-$Date


#if [ $Date != $Today ] ; then
  if [ -z "$doReload" -a -z "$isCSV" -a -f ~/htdocs/tmp/$Bmp.gif ] ; then
    CacheDate=`stat -c %y ~/htdocs/tmp/$Bmp.gif | cut -b 1-10`
    # this will fail on the same day, and for any cached data that was done the same day (i.e. incomplete)
    if [ $CacheDate \> $Date ] ; then
      echo "N/A: cached image from " `stat -c %y ~/htdocs/tmp/$Bmp.gif | cut -b 1-19`
      exit
    fi
  fi
#fi

# find log file for $Date
gdate=$Date
i=0;
while true; do
  if [ "$gdate" != `date +%F` ] ; then
    ldate=`date -d "$gdate+1 days" +%F`
    [ $? != 0 ] && exit;
    if [ -f ~/var/logs.dated/*/*$gdate.23:59.$FILE0.gz ] ; then
      FILE=~/var/logs.dated/*/*$gdate.23:59.$FILE0.gz
    else
      FILE=~/var/logs.dated/*/*$ldate.00:00.$FILE0.gz
    fi
    CAT=zcat
    [ -f $FILE ] && break;
    gdate=$ldate 
  else
    gdate=`date +%F`
    if [ -f ~/var/log/apache/$FILE0 ] ; then
      FILE=~/var/log/apache/$FILE0
    else
      FILE=~/var/log/$FILE0
    fi
    CAT=cat
    break
  fi
  # make sure no infinite loop happens
  i=`expr $i + 1`
  [ $i -gt 14 ] && break;
done


if [ ! -f $FILE ] ; then
  echo "No data for $Date";
  exit
fi
if [ -z "$isCSV" ] ; then
  $CAT $FILE | $POSTPROC -d $Date | $DIR/grapher ~/htdocs/tmp/$Bmp.gif.$$ $@
  mv -f ~/htdocs/tmp/$Bmp.gif.$$  ~/htdocs/tmp/$Bmp.gif
else
  flds=`echo $@ | sed -e "s/ /:/g"`
  #echo $flds;
  $CAT $FILE | $POSTPROC -d $Date | awk -vfields=$flds -v date="$Date" -f $DIR/makecsv.awk 2>&1
fi
