#!/bin/bash
#
# Checks and waits for the queue to be emptied
#
# waitForQueueTask.sh -q <queue>... [-o <progOK>] [-f <progFail>] [-O <param>]... [-F <param>] [-R <maxRetry>]
#
#  -o <progOK>    : start this program if the queue is empty
#  -f <progFail>  : start this program if the queue is not empty and timed out
#  -O <param>     : pass this parameter(s) to progOK
#  -F <param>     : pass this parameter(s) to progFail
#  -R <maxRetry>  : maximum number of retries (default: 10)
#
# Example:
#   isQueueEmpty.sh -o continueTest -f stopTest -p -c -p <configfilename>
#
#
# Last Updated     : $Author$
# Update Date Time : $Date$
# RCS File Name    : $RCSfile$
# RCS Revision     : $Revision$
# Locked by        : $Locker$
# Symbolic name    : $Name$
# State            : $State$
#

. $HOME/etc/setenv.sh

# if waitForQueueTask.sh is called directly:
# run as a task instead
if [ `basename $0` = "waitForQueueTask.sh" ] ; then
	$TASKD_HOME/runshellcmd.sh -R -r 0 $0 "$@"
	exit $?
fi


OKSCRIPT=""
FAILSCRIPT=""

SELFNAME=`basename $0`

PASSOKPARAMS=""
PASSFAILPARAMS=""

MAXRETRY="10"


while getopts 'o:f:O:F:R:' OPT; do
	case $OPT in
	o) OKSCRIPT="$OPTARG" ;;
	f) FAILSCRIPT="$OPTARG" ;;
	O) PASSOKPARAMS="$PASSOKPARAMS \"$OPTARG\"" ;;
	F) PASSFAILPARAMS="$PASSFAILPARAMS \"$OPTARG\"" ;;
	R) MAXRETRY="$OPTARG" ;;
	*) ERRMSG="Invalid_Parameter:$OPT" ;;
	esac
done


check_queue_()
{
	# first check the queue's root
	[ `ls $1 | wc -l` -ne 0 ] && return 1
	
	# check the retry queue
	[ `ls $1/.retry | wc -l` -ne 0 ] && return 1
	
	# check the scheduled queue
	[ `ls $1/.scheduled | wc -l` -gt 1 ] && return 1
	
	TASKNAME=`ls $1/.scheduled`
	TNAME=`[ -n "$TASKNAME" ] && awk '{n = split($1,arr,"/"); print arr[n]}' $TASKNAME`

	[ -n "$TNAME" -a "$TNAME" != "$SELFNAME" ] && return 1
	
	return 0
}


run_() # override run_ method
{
	
	check_queue_ $TASKD_GLOBALQUEUE
	[ $? -ne 0 ] && return 1

	sleep 1
	
	check_queue_ $TASKD_GLOBALQUEUE
	[ $? -ne 0 ] && return 1

	[ -n "$OKSCRIPT" ] && eval "$OKSCRIPT $PASSOKPARAMS"

	return 0
}


print_()
{
	echo "Waits for the queues to be emptied"
}

calculate_retry_min_()
{
	[ -z "$RETRY_NUM" ] && RETRY_NUM=0

	if [ $RETRY_NUM -gt $MAXRETRY ] ; then
		[ -n "$FAILSCRIPT" ] && eval "$FAILSCRIPT $PASSFAILPARAMS"
		return 125
	fi

	return 1    #  1 min
}

